@@ -1,7 +1,7 @@
 # -*- coding: UTF-8 -*-
 """Expand seeds into dependency-closed lists of packages."""
 
-# Copyright (c) 2004, 2005 Canonical Ltd.
+# Copyright (c) 2004, 2005, 2006, 2007 Canonical Ltd.
 #
 # Germinate is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -69,10 +69,11 @@ class Germinator:
 
         self.blacklist = {}
         self.blacklisted = set()
+        self.seedblacklist = {}
 
         self.di_kernel_versions = {}
-        self.extra_includes = {}
-        self.extra_excludes = {}
+        self.includes = {}
+        self.excludes = {}
 
     def debug(self, msg, *args, **kwargs):
         logging.debug(msg, *args, **kwargs)
@@ -245,9 +246,10 @@ class Germinator:
         self.build_srcs[seedname] = set()
         self.not_build_srcs[seedname] = set()
         self.why[seedname] = {}
+        self.seedblacklist[seedname] = set()
         self.di_kernel_versions[seedname] = set()
-        self.extra_includes[seedname] = []
-        self.extra_excludes[seedname] = []
+        self.includes[seedname] = {}
+        self.excludes[seedname] = {}
 
     def filterPackages(self, packages, pattern):
         """Filter a list of packages, returning those that match the given
@@ -319,6 +321,12 @@ class Germinator:
                 outerseeds.append(seed)
         return outerseeds
 
+    def outerSeeds(self, seedname):
+        """Return this seed and the seeds that inherit from it."""
+        outerseeds = [seedname]
+        outerseeds.extend(self.strictlyOuterSeeds(seedname))
+        return outerseeds
+
     def alreadySeeded(self, seedname, pkg):
         """Has pkg already been seeded in this seed or in one from
         which we inherit?"""
@@ -358,12 +366,30 @@ class Germinator:
                     # Allows us to pick the right modules later
                     self.warning("Allowing d-i kernel versions: %s", values)
                     self.di_kernel_versions[seedname].update(values)
-                elif name == "extra-include":
-                    self.warning("Including packages from extra: %s", values)
-                    self.extra_includes[seedname].extend(values)
-                elif name == "extra-exclude":
-                    self.warning("Excluding packages from extra: %s", values)
-                    self.extra_excludes[seedname].extend(values)
+                elif name.endswith("-include"):
+                    included_seed = name[:-8]
+                    if (included_seed not in self.seeds and
+                        included_seed != "extra"):
+                        self.error("Cannot include packages from unknown "
+                                   "seed: %s", included_seed)
+                    else:
+                        self.warning("Including packages from %s: %s",
+                                     included_seed, values)
+                        if included_seed not in self.includes[seedname]:
+                            self.includes[seedname][included_seed] = []
+                        self.includes[seedname][included_seed].extend(values)
+                elif name.endswith("-exclude"):
+                    excluded_seed = name[:-8]
+                    if (excluded_seed not in self.seeds and
+                        excluded_seed != "extra"):
+                        self.error("Cannot exclude packages from unknown "
+                                   "seed: %s", excluded_seed)
+                    else:
+                        self.warning("Excluding packages from %s: %s",
+                                     excluded_seed, values)
+                        if excluded_seed not in self.excludes[seedname]:
+                            self.excludes[seedname][excluded_seed] = []
+                        self.excludes[seedname][excluded_seed].extend(values)
                 self.substvars[name] = values
                 continue
 
@@ -375,11 +401,24 @@ class Germinator:
                 if pkg.endswith("]"):
                     archspec = pkg[startarchspec + 1:-1].split()
                     pkg = pkg[:startarchspec - 1]
-                    if arch not in archspec:
+                    posarch = [x for x in archspec if not x.startswith('!')]
+                    negarch = [x[1:] for x in archspec if x.startswith('!')]
+                    if arch in negarch:
+                        continue
+                    if posarch and arch not in posarch:
                         continue
 
             pkg = pkg.split()[0]
 
+            # a leading ! indicates a per-seed blacklist; never include this
+            # package in the given seed or any of its inner seeds, no matter
+            # what
+            if pkg.startswith('!'):
+                pkg = pkg[1:]
+                is_blacklist = True
+            else:
+                is_blacklist = False
+
             # a (pkgname) indicates that this is a recommend
             # and not a depends
             if pkg.startswith('(') and pkg.endswith(')'):
@@ -403,8 +442,14 @@ class Germinator:
                 if not pkgs:
                     pkgs = [pkg] # virtual or expanded; check again later
 
-            for pkg in pkgs:
-                seedpkgs.extend(self.substituteSeedVars(pkg))
+            if is_blacklist:
+                for pkg in pkgs:
+                    self.info("Blacklisting %s from %s", pkg, seedname)
+                    self.seedblacklist[seedname].update(
+                        self.substituteSeedVars(pkg))
+            else:
+                for pkg in pkgs:
+                    seedpkgs.extend(self.substituteSeedVars(pkg))
 
         for pkg in seedpkgs:
             if pkg in self.hints and self.hints[pkg] != seedname:
@@ -468,19 +513,53 @@ class Germinator:
                 if self.is_pruned(pkg, seed):
                     self.pruned[pkg].add(seed)
 
+    def weedBlacklist(self, pkgs, seedname, build_tree, why):
+        """Weed out blacklisted seed entries from a list."""
+        white = []
+        if build_tree:
+            outerseeds = ["supported"]
+        else:
+            outerseeds = self.outerSeeds(seedname)
+        for pkg in pkgs:
+            for outerseed in outerseeds:
+                if (outerseed in self.seedblacklist and
+                    pkg in self.seedblacklist[outerseed]):
+                    self.error("Package %s blacklisted in %s but seeded in "
+                               "%s (%s)", pkg, outerseed, seedname, why)
+                    break
+            else:
+                white.append(pkg)
+        return white
+
     def grow(self):
         """Grow the seeds."""
         for seedname in self.seeds:
             self.progress("Resolving %s dependencies ...", seedname)
+            if self.seedrelease[seedname] is None:
+                why = "%s seed" % seedname.title()
+            else:
+                why = ("%s %s seed" %
+                       (self.seedrelease[seedname].title(), seedname))
+
+            # Check for blacklisted seed entries.
+            self.seed[seedname] = self.weedBlacklist(
+                self.seed[seedname], seedname, False, why)
+            self.seedrecommends[seedname] = self.weedBlacklist(
+                self.seedrecommends[seedname], seedname, False, why)
+
             for pkg in self.seed[seedname] + self.seedrecommends[seedname]:
-                if self.seedrelease[seedname] is None:
-                    self.addPackage(seedname, pkg, "%s seed" %
-                        seedname.title())
-                else:
-                    self.addPackage(seedname, pkg, "%s %s seed" %
-                        (self.seedrelease[seedname].title(), seedname))
-            self.rescueExtras(seedname, build_tree=False)
-        self.rescueExtras("supported", build_tree=True)
+                self.addPackage(seedname, pkg, why)
+
+            for rescue_seedname in self.seeds:
+                self.rescueIncludes(seedname, rescue_seedname,
+                                    build_tree=False)
+                if rescue_seedname == seedname:
+                    # only rescue from seeds up to and including the current
+                    # seed; later ones have not been grown
+                    break
+            self.rescueIncludes(seedname, "extra", build_tree=False)
+
+        self.rescueIncludes("supported", "extra", build_tree=True)
 
     def addExtras(self, seedrelease=None):
         """Add packages generated by the sources but not in any seed."""
@@ -676,17 +755,21 @@ class Germinator:
                            virtual, pkg)
                 return False
 
+        if build_tree and build_depend:
+            why = self.packages[pkg]["Source"] + " (Build-Depend)"
+        else:
+            why = pkg
+
+        dependlist = self.weedBlacklist(dependlist, seedname, build_tree, why)
+        if not dependlist:
+            return False
+
         if build_tree:
             for dep in dependlist:
                 self.build_depends[seedname].add(dep)
-            if build_depend:
-                why = self.packages[pkg]["Source"] + " (Build-Depend)"
-            else:
-                why = pkg
         else:
             for dep in dependlist:
                 self.depends[seedname].add(dep)
-            why = pkg
 
         for dep in dependlist:
             self.addPackage(seedname, dep, why, build_tree, second_class)
@@ -736,6 +819,16 @@ class Germinator:
         if seedname in self.pruned[pkg]:
             self.warning("Pruned %s from %s", pkg, seedname)
             return
+        if build_tree:
+            outerseeds = ["supported"]
+        else:
+            outerseeds = self.outerSeeds(seedname)
+        for outerseed in outerseeds:
+            if (outerseed in self.seedblacklist and
+                pkg in self.seedblacklist[outerseed]):
+                self.error("Package %s blacklisted in %s but seeded in %s "
+                           "(%s)", pkg, outerseed, seedname, why)
+                return
         if build_tree: second_class=True
 
         if pkg not in self.all:
@@ -814,39 +907,52 @@ class Germinator:
                                self.sources[src]["Build-Depends-Indep"],
                                build_depend=True)
 
-    def rescueExtras(self, seedname, build_tree):
+    def rescueIncludes(self, seedname, rescue_seedname, build_tree):
         """Automatically rescue packages matching certain patterns from
-        extra."""
+        other seeds."""
+
+        if seedname not in self.seeds or rescue_seedname not in self.seeds:
+            return
 
         # Find all the source packages.
-        extra_srcs = set()
-        for seed in self.innerSeeds(seedname):
+        rescue_srcs = set()
+        if rescue_seedname == "extra":
+            rescue_seeds = self.innerSeeds(seedname)
+        else:
+            rescue_seeds = [rescue_seedname]
+        for seed in rescue_seeds:
             if build_tree:
-                extra_srcs |= self.build_srcs[seed]
+                rescue_srcs |= self.build_srcs[seed]
             else:
-                extra_srcs |= self.not_build_srcs[seed]
+                rescue_srcs |= self.not_build_srcs[seed]
 
         # For each source, add any binaries that match the include/exclude
         # patterns.
-        for src in extra_srcs:
-            extras = [p for p in self.sources[src]["Binaries"]
+        for src in rescue_srcs:
+            rescue = [p for p in self.sources[src]["Binaries"]
                         if p in self.packages]
-            included_extras = set()
-            for include in self.extra_includes[seedname]:
-                included_extras |= set(self.filterPackages(extras, include))
-            for exclude in self.extra_excludes[seedname]:
-                included_extras -= set(self.filterPackages(extras, exclude))
-            for pkg in included_extras:
+            included = set()
+            if (seedname in self.includes and
+                rescue_seedname in self.includes[seedname]):
+                for include in self.includes[seedname][rescue_seedname]:
+                    included |= set(self.filterPackages(rescue, include))
+            if (seedname in self.excludes and
+                rescue_seedname in self.excludes[seedname]):
+                for exclude in self.excludes[seedname][rescue_seedname]:
+                    included -= set(self.filterPackages(rescue, exclude))
+            for pkg in included:
                 if pkg in self.all:
                     continue
                 for lesserseed in self.strictlyOuterSeeds(seedname):
                     if pkg in self.seed[lesserseed]:
                         self.seed[lesserseed].remove(pkg)
                         self.warning("Promoted %s from %s to %s due to "
-                                     "Extra-Includes",
-                                     pkg, lesserseed, seedname)
+                                     "%s-Includes",
+                                     pkg, lesserseed, seedname,
+                                     rescue_seedname.title())
                         break
-                self.debug("Rescued %s from extra to %s", pkg, seedname)
+                self.debug("Rescued %s from %s to %s", pkg,
+                           rescue_seedname, seedname)
                 if build_tree:
                     self.build_depends[seedname].add(pkg)
                 else:
