@@ -1,10 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
-# arch-tag: e7f7b26b-95bc-4432-86e6-832a7cc5ac01
-"""Update list files from the Wiki.
-"""
+"""Update list files from the Wiki."""
 
-# Copyright (c) 2004, 2005 Canonical Ltd.
+# Copyright (c) 2004, 2005, 2006, 2007 Canonical Ltd.
 #
 # Germinate is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -21,7 +19,6 @@
 # Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-import apt_pkg
 import gzip
 import os
 import shutil
@@ -31,9 +28,6 @@ import getopt
 import logging
 import codecs
 import cStringIO
-from Germinate import Germinator
-import Germinate.Archive
-import Germinate.seeds
 
 try:
     set # introduced in 2.4
@@ -41,17 +35,24 @@ except NameError:
     import sets
     set = sets.Set
 
+import apt_pkg
+
+from Germinate import Germinator
+import Germinate.Archive
+import Germinate.seeds
+import Germinate.version
+
 
 # Where do we get up-to-date seeds from?
-SEEDS = "http://people.ubuntu.com/~cjwatson/seeds/"
+SEEDS = "http://people.ubuntu.com/~ubuntu-archive/seeds/"
 SEEDS_BZR = "http://bazaar.launchpad.net/~ubuntu-core-dev/ubuntu-seeds/"
-RELEASE = "ubuntu.feisty"
+RELEASE = "ubuntu.gutsy"
 
 # If we need to download Packages.gz and/or Sources.gz, where do we get
 # them from?
 MIRROR = "http://archive.ubuntu.com/ubuntu/"
 SOURCE_MIRROR = None
-DIST = ["feisty"]
+DIST = ["gutsy"]
 COMPONENTS = ["main", "restricted"]
 ARCH = "i386"
 
@@ -254,7 +255,8 @@ def usage(f):
 
 Options:
 
-  -h, --help            Print this help message.
+  -h, --help            Print this help message and exit.
+  --version             Output version information and exit.
   -v, --verbose         Be more verbose when processing seeds.
   -S, --seed-source=SOURCE
                         Fetch seeds from SOURCE
@@ -294,6 +296,7 @@ def main():
     try:
         opts, args = getopt.getopt(sys.argv[1:], "hvS:s:m:d:c:a:i",
                                    ["help",
+                                    "version",
                                     "verbose",
                                     "seed-source=",
                                     "seed-dist=",
@@ -315,6 +318,10 @@ def main():
         if option in ("-h", "--help"):
             usage(sys.stdout)
             sys.exit()
+        elif option == "--version":
+            print "%s %s" % (os.path.basename(sys.argv[0]),
+                             Germinate.version.VERSION)
+            sys.exit()
         elif option in ("-v", "--verbose"):
             verbose = True
         elif option in ("-S", "--seed-source"):
