@@ -1,8 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
-# arch-tag:
 
-# Copyright (c) 2004 Canonical Ltd.
+# Copyright (c) 2004, 2005, 2006, 2007 Canonical Ltd.
 #
 # This file is part of Germinate.
 #
@@ -21,18 +20,23 @@
 # Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 # 02110-1301, USA.
 
-import os, sys, getopt
+import os
+import sys
+import getopt
 import logging
+
 import apt_pkg
+
 from Germinate import Germinator
 import Germinate.Archive
 import Germinate.seeds
+import Germinate.version
 
 # TODO: cloned from germinate.py; should be common
-SEEDS = "http://people.ubuntu.com/~cjwatson/seeds/"
-RELEASE = "ubuntu.feisty"
+SEEDS = "http://people.ubuntu.com/~ubuntu-archive/seeds/"
+RELEASE = "ubuntu.gutsy"
 MIRROR = "http://archive.ubuntu.com/ubuntu/"
-DIST = ["feisty"]
+DIST = ["gutsy"]
 COMPONENTS = ["main"]
 ARCH = "i386"
 
@@ -96,20 +100,29 @@ class Globals:
         apt_pkg.InitConfig()
         apt_pkg.Config.Set("APT::Architecture", ARCH)
 
-        Germinate.Archive.TagFile(MIRROR).feed(g, DIST, COMPONENTS, ARCH)
+        Germinate.Archive.TagFile(MIRROR).feed(g, DIST, COMPONENTS, ARCH, True)
 
         (seednames, seedinherit) = g.parseStructure(
             Germinate.seeds.open_seed(SEEDS + RELEASE, "STRUCTURE"))
+        needed_seeds = []
         for seedname in self.seeds:
+            for inherit in seedinherit[seedname]:
+                if inherit not in needed_seeds:
+                    needed_seeds.append(inherit)
+            needed_seeds.append(seedname)
+        for seedname in needed_seeds:
             g.plantSeed(Germinate.seeds.open_seed(SEEDS + RELEASE, seedname),
                         ARCH, seedname, list(seedinherit[seedname]), RELEASE)
         g.prune()
         g.grow()
 
-        for seedname in self.seeds:
+        for seedname in needed_seeds:
             for pkg in g.seed[seedname]:
                 self.package.setdefault(pkg, Package(pkg))
                 self.package[pkg].setSeed(seedname + ".seed")
+            for pkg in g.seedrecommends[seedname]:
+                self.package.setdefault(pkg, Package(pkg))
+                self.package[pkg].setSeed(seedname + ".seed-recommends")
             for pkg in g.depends[seedname]:
                 self.package.setdefault(pkg, Package(pkg))
                 self.package[pkg].setSeed(seedname + ".depends")
@@ -143,23 +156,29 @@ def usage(f):
 
 Options:
 
-  -h, --help            Print this help message.
+  -h, --help            Print this help message and exit.
+  --version             Output version information and exit.
   -l, --list=FILE       Read list of packages from this file
-                        (default: read from dpkg --get-selections)
+                        (default: read from dpkg --get-selections).
   -m, --mode=[i|r|d]    Show packages to install/remove/diff (default: d).
+  -a, --arch=ARCH       Operate on architecture ARCH (default: %s).
 
 A list of seeds against which to compare may be supplied as non-option
-arguments. The default is 'minimal standard desktop'.
-"""
+arguments. Seeds from which they inherit will be added automatically. The
+default is 'desktop'.
+""" % ARCH
 
 def main():
+    global ARCH
     g = Globals()
 
     try:
-        opts, args = getopt.getopt(sys.argv[1:], "hl:m:",
+        opts, args = getopt.getopt(sys.argv[1:], "hl:m:a:",
                                    ["help",
+                                    "version",
                                     "list=",
-                                    "mode="])
+                                    "mode=",
+                                    "arch="])
     except getopt.GetoptError:
         usage(sys.stderr)
         sys.exit(2)
@@ -169,15 +188,21 @@ def main():
         if option in ("-h", "--help"):
             usage(sys.stdout)
             sys.exit()
+        elif option == "--version":
+            print "%s %s" % (os.path.basename(sys.argv[0]),
+                             Germinate.version.VERSION)
+            sys.exit()
         elif option in ("-l", "--list"):
             dpkgFile = value
-        elif option in ("-m", "--mirror"):
+        elif option in ("-m", "--mode"):
             # one of 'i' (install), 'r' (remove), or 'd' (default)
             g.setOutput(value)
+        elif option in ("-a", "--arch"):
+            ARCH = value
 
     g.parseDpkg(dpkgFile)
     if not len(args):
-        args = ["minimal", "standard", "desktop"]
+        args = ["desktop"]
     g.setSeeds(args)
     g.output()
 
